/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include <cstdlib>
#include <limits.h>
#include "log.h"
#include "rebus.h"
#include "misc.h"
#include "cur.h"



void Rebus::SetCur(Cur * pcur)
{
	cur = pcur;
}


bool Rebus::InitPair(int a, int b, Item & item)
{
wchar_t buffer[100];
size_t buffer_len = sizeof(buffer) / sizeof(wchar_t);
bool add = false;

	if( a+b <= 15 )
	{
		swprintf(buffer, buffer_len, L"%d+%d", a, b);
		item.question = buffer;
		item.answer = a+b;
		add = true;
	}

	if( a-b >= 0 )
	{
		swprintf(buffer, buffer_len, L"%d-%d", a, b);
		item.question = buffer;
		item.answer = a-b;
		add = true;
	}

return add;
}



void Rebus::Init()
{
int a, b;
Item item;

	item.key = 0;

	for(a=1 ; a<10 ; ++a)
	{
		for(b=1 ; b<10 ; ++b)
		{
			if( InitPair(a, b, item) )
			{
				table.push_back(item);
				item.key += 1;
			}
		}
	}
}


Rebus::Item * Rebus::Rand()
{
size_t len = table.size();

	if( len == 0 )
		// empty table, call Init() first
		return 0;

	size_t i = size_t( ((double)rand()*(double)len) / (double(RAND_MAX)+1.0) );

	if( i >= table.size() )
	{
		log << log1 << "Rebus: Rand() index too big" << logend;
		// oops, it should not be greater then table.size()
		return &table[0];
	}

return &table[0] + i;
}



bool Rebus::IsAnswerOk(Rebus::Item * item, const std::wstring & answer)
{
	if( item == 0 )
		return false;

	wchar_t * end;
	const wchar_t * a = answer.c_str();
	a = SkipWhite(a);

	int value = (int)wcstol(a, &end, 10);

	if( a == end )
		// nothing has been read
		return false;

	log << log2 << "Rebus: your answer is: " << value << logend;
	end = (wchar_t*)SkipWhite(end);

	if( *end != 0 )
	{
		// something is at the end
		return false;
	}

	bool result = (item->answer == value);

	if( result )
		log << log3 << "Rebus: the answer is correct" << logend;

return result;
}



bool Rebus::CheckRebus()
{
	if( !cur->session )
		return false;

	if( cur->session->puser )
		// logged users don't have to use the rebus
		return true;

	if( cur->session->rebus_checked )
		return true;

	cur->session->rebus_checked = true;

	if( !cur->session->rebus_item )
	{
		log << log1 << "Rebus: rebus not set" << logend;
		return false;
	}

	if( IsAnswerOk(cur->session->rebus_item, cur->request->PostVar(L"rebus")) )
		return true;

	log << log1 << "Rebus: rebus has an incorrect answer, expected: "
		<< cur->session->rebus_item->answer << logend;

	// don't add cur->session->spam_score when the rebus has incorrect answer
	// a user could have made a mistake

return false;
}











