/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2012, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfile_winix_core_session
#define headerfile_winix_core_session

#include <vector>
#include <string>
#include <ctime>
#include "item.h"
#include "error.h"
#include "user.h"
#include "plugindata.h"
#include "rebus.h"
#include "textstream.h"
#include "date/date.h"
#include "ipban.h"



struct Session
{
	Session();
	Session(const Session & ses);
	Session & operator=(const Session & ses);

	void SetTimesTo(time_t time);
	void Clear(bool clear_plugin_data = true);
	void ClearOnEndRequest();


	// 0 - means that there is a temporary session
	long id;
	
	// true if the session was created now
	bool new_session;

	// when this session was created
	// (the same values)
	time_t   start_time;
	PT::Date start_date;
	
	// when this session was last used
	// (the same values)
	time_t   last_time;
	PT::Date last_date;

	// when there was a last get request
	// (used to calculate spam or invalid login attempts etc.)
	time_t last_time_get;
	

	// 0 - means that nobody is logged	
	User * puser;

	// if false the session will end when the user browser is shutdown
	bool remember_me;


	// rebus - set by rebus_question(Info & i) from templates
	Rebus::Item * rebus_item;
	bool rebus_checked;


	int spam_score;

	// if true then this session will be removed by SessionManager
	// without checking the time expiration
	bool remove_me;

	PluginData plugin_data;


	// buffer for the session log
	TextStream<std::wstring> log_buffer;

	// !! IMPROVE ME it is still needed?
	// css cannot be taken directly from the mountpoint?
	// table with css files
	// used by some javascript wysiwyg editors (ckeditor, tinymce)
	std::vector<std::wstring> last_css;

	// pointer to IPBan struct if exists for this IP
	// many sessions can pointer to the same IPBan struct
	// (it can be null)
	IPBan * ip_ban;

};




#endif

