/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010-2012, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfile_winix_db_dbbase
#define headerfile_winix_db_dbbase


#include "dbconn.h"
#include "dbtextstream.h"
#include <vector>
#include <string>
#include "core/error.h"
#include "space/spaceparser.h"
#include "date/date.h"



class DbBase
{
public:

	DbBase();

	void SetConn(DbConn * conn);
	void SetConn(DbConn & conn);
	DbConn * GetConn();

	void LogQueries(bool log_q);

	PGresult * AssertQuery(const char * query);
	PGresult * AssertQuery(const DbTextStream & query);
	PGresult * AssertQuery(const char * q, ExecStatusType t);
	PGresult * AssertQuery(const DbTextStream & query, ExecStatusType t);
	void AssertResult(PGresult * r, ExecStatusType t);
	static int AssertColumn(PGresult * r, const char * column_name);

	// static assignments
	static const char *  AssertValue(PGresult * r, int row, int col);
	static const std::wstring & AssertValueWide(PGresult * r, int row, int col); // warning: this method uses a static buffer
	static void          AssertValueWide(PGresult * r, int row, int col, std::wstring & result);
	static void          AssertValueBin(PGresult * r, int row, int col, std::string & result);
	static long          AssertValueLong(PGresult * r, int row, int col);
	static int           AssertValueInt(PGresult * r, int row, int col);
	static bool          AssertValueBool(PGresult * r, int row, int col);
	static unsigned long AssertValueULong(PGresult * r, int row, int col);
	static unsigned int  AssertValueUInt(PGresult * r, int row, int col);
	static PT::Date      AssertValueDate(PGresult * r, int row, int col);
	
	// non static assignments
	bool AssertValueSpace(PGresult * r, int row, int col, PT::Space & space, bool split_single = true);

	void ClearResult(PGresult * r);
	long AssertCurrval(const char * table);

	bool IsNull(PGresult * r, int row, int col);
	int Rows(PGresult * r);
	int Cols(PGresult * r);
	long AffectedRows(PGresult * r);

	void CreateIdList(const std::vector<long> & id_tab, std::wstring & list, bool add_parentheses = true);

	Error DoCommand(const DbTextStream & command);
	Error DoCommand(const char * command);
	Error BeginTrans();
	Error RollbackTrans();
	Error CommitTrans();
	Error EndTrans(Error err);

	// new API (returns only bool)
	bool EndTrans(bool everything_ok);

	static void UnescapeBin(const char * str, size_t len, std::string & out, bool clear_out = true);


protected:

	// a helper method for escaping strings
	template<class RawType>
	DbTextStream::RawText<RawType> R(const RawType & par)
	{
		return DbTextStream::RawText<RawType>(par);
	}

	DbConn * db_conn;
	bool log_queries;


private:

//	static int  CharToInt(char c);
//	static bool IsCorrectOctalDigit(char c);
//	static int  UnescapeBin(const char * str, size_t & i, size_t len);

	static char UnescapeBinHexToDigit(char hex);

	DbTextStream bquery;
	PT::SpaceParser conf_parser;

};




#endif

